// Format tohoto generatoru neni nutne dodrzovat nejak striktne, ale zavadi
// rozumne konvence. Pokud necemu v kodu nerozumis, neboj se nekoho se zeptat
// (autorem vzoru je Vasek Volhejn)

#include <algorithm>
#include <cassert>
#include <fstream>
#include <iomanip>
#include <iostream>
#include <random>
#include <sstream>
#include <string>
#include <unordered_set>
#include <vector>
#include <math.h>

using namespace std;
#define rep(i, a, n) for (int i = a; i < (int)(n); i++)

// rand() nepouzivame, napr. protoze neni definovane, jak velka cisla muze vracet
#define rand(x) ("Misto rand() prosim pouzivej funkce rand* definovane zde", +)
#define ll long long
using namespace std;

#define INF std::numeric_limits<int>::max()
// Generator nahodnych cisel
std::mt19937_64 rng;

ll randRange(ll from, ll to) {
    to = max(to, from);
    // Vrati nahodny long long v intervalu [from, to] (tj. vcetne `to`).
    std::uniform_int_distribution<ll> dist(from, to);
    return dist(rng);
}

double randDouble(double from, double to) {
    // Vrati nahodny double v intervalu [from, to), (tj. bez `to`).
    std::uniform_real_distribution<double> dist(from, to);
    return dist(rng);
}

ll randAlmost(ll to, double min_part = 0.9) {
    // Vrati nahodny long long ktery je v nejakem smyslu "skoro tak velky" jako `to`,
    // konkretne vrati cislo v intervalu [floor(value*min_part), value].
    // Uzitecne napr. pro generovani velikosti problemu
    if (to == 0)
        return 0;
    assert(to >= 1);
    return randRange(min_part * to, to);
}

bool randBool(double p = 0.5) {
    // Vrati true s pravdepodobnosti p a false s pravdepodobnosti 1-p
    return randDouble(0, 1) <= p;
}

vector<ll> randDifferentNumbers(ll from, ll to, int k) {
    // Vrati k ruznych nahodnych cisel z intervalu [from, to]
    // Implementace podle: https://www.gormanalysis.com/blog/random-numbers-in-cpp/

    unordered_set<ll> samples;
    ll n = to - from + 1;
    assert(n >= k);

    for (ll r = n - k; r < n; ++r) {
        ll v = uniform_int_distribution<ll>(from, from + r)(rng);
        if (!samples.insert(v).second)
            samples.insert(from + r);
    }

    vector<ll> result(samples.begin(), samples.end());
    shuffle(result.begin(), result.end(), rng);

    return result;
}

/////////////////////////////////////////////////////////////
// konec náhodných čísel a náhodných věcí
/////////////////////////////////////////////////////////////

// TODO: upravit tuto ukazku pro svou ulohu

// Pro konstanty, ktere se lisi mezi obtiznostmi, muzeme pouzit takovyto format;
// MAX_N[diff] rika, jake je maximalni N pro danou obtiznost (0 nebo 1)
const int SUBTASKS = 3;
const ll MIN_N = 1;
const ll MAX_N[3] = {(int) 1e3, (int) 1e9, (int) 1e9};

const int MIN_Q = 2;
const int MAX_Q[3] = {(int) 1e3, (int) 2e5, (int) 2e5};
const bool SHORT[3] = {false, true, false};
const ll SHORT_LENGTH = 10;

struct Problem {
    // Reprezentuje jeden problem (test).
    ll n;
    int q;
    vector<pair<ll, ll>> queries;

    string name;  // Jen interni pojmenovani

    Problem(string _name) : name(_name) {}

    void check(int subtask) {
        // Tato metoda neni nutna, ale je uzitecna jako zakladni kontrola:
        // zkontroluje, jestli problem odpovida limitum stanovenym v zadani
        assert(MIN_N <= n && n <= MAX_N[subtask]);
        assert(MIN_Q <= q && q <= MAX_Q[subtask] && q == (int) queries.size());

        for (auto query: queries) {
            assert(0 <= query.first && query.first <= query.second && query.second <= n);
            if (subtask == 1) assert(query.second - query.first <= 10);
        }
    }

    void print(ofstream& of) {
        // Vypise problem tak, jak to specifikuje sekce "Tvar vstupu"
        // (nezapomente napriklad nemit nekde prebytecne mezery)
        of << n << " " << q << endl;
        for (auto query: queries) {
            of << query.first << " " << query.second << endl;
        }
    }
};

Problem gen_samle1() {
    Problem p("sample");
    p.n = 6; p.q = 4;
    p.queries = {
        {0, 2},
        {1, 3},
        {5, 6},
        {4, 4},
    };
    return p;
}

Problem gen_rand(ll n, int q, int subtask) {
    Problem problem("rand");
    problem.n = n; problem.q = q;
    ll length = SHORT[subtask] ? SHORT_LENGTH : n / q * 100;
    for (int i=0; i<q; i++) {
        ll a = randRange(0, n);
        ll b = randRange(a - length, a + length);
        b = min(max(MIN_N, b), n);
        problem.queries.push_back({min(a, b), max(a, b)});
    }
    return problem;
}

Problem gen_merge(ll n, int q, int subtask) {
    Problem problem("merge");
    q = min(n, (ll) q);
    problem.n = n; problem.q = q;
    vector<pair<ll, ll>> even, odd;
    ll length = max((ll) 1, n / (q+1));
    if (SHORT[subtask]) length = min(length, 9*SHORT_LENGTH/10);
    for (int i=0; i<q; i++) {
        int shift = randRange(-length/10, length/10);
        pair<ll, ll> x = {
            i*length     + length/10 + shift,
            (i+1)*length + length/5  + shift
        };
        if (i%2) odd.push_back(x);
        else    even.push_back(x);
    }
    shuffle(even.begin(), even.end(), rng);
    shuffle(odd.begin(), odd.end(), rng);
    problem.queries = even;
    for (auto x: odd) problem.queries.push_back(x);
    shuffle(problem.queries.begin() + q*3/7, problem.queries.begin() + q*4/7, rng);
    return problem;
}

Problem gen_converge(ll n, int q, int subtask) {
    Problem problem("converge");
    q = min(n, (ll) q);
    problem.n = n; problem.q = q;
    vector<pair<ll, ll>> intervals;
    ll length = max((ll) 1, n / (q+1) / 2);
    if (SHORT[subtask]) length = min(length, 5*SHORT_LENGTH/10);
    for (int i=0; i<q; i++) {
        int shift = randRange(-length/5, length/5);
        pair<ll, ll> x = {
            i*length       + (1+2*i)*length/5 + i + shift,
            (i+1)*(length) + (1+2*i)*length/5 + i + shift
        };
        intervals.push_back(x);
    }
    for(int i=0; i<q/2; i++) {
        problem.queries.push_back(intervals[i]);
        problem.queries.push_back(intervals[q-i-1]);
    }
    if (q % 2) problem.queries.push_back(intervals[q/2]);

    int sr = 1000;
    for (int i=0; i<sr; i++) {
        shuffle(problem.queries.begin() + q*i/sr, problem.queries.begin() + q*(i+1)/sr, rng);
    }
    return problem;
}

int main() {
    srand(5603);
    rng.seed(7895);

    rep(si, 0, SUBTASKS) {
        vector<Problem> tests;
        /* if (si != 1 && si != 3)
            tests.push_back(gen_samle1());
        if (si == 6)
            tests.push_back(gen_samle2()); */
        tests.push_back(gen_samle1());
        tests.push_back(gen_rand(MIN_N, MAX_Q[si], si));

        tests.push_back(gen_rand(MAX_N[si], MAX_Q[si], si));
        tests.push_back(gen_rand(randAlmost(MAX_N[si]), randAlmost(MAX_Q[si]), si));
        tests.push_back(gen_rand(randAlmost(MAX_N[si]), randAlmost(MAX_Q[si]), si));
        
        tests.push_back(gen_merge(MAX_N[si], MAX_Q[si], si));
        tests.push_back(gen_merge(randAlmost(MAX_N[si]), randAlmost(MAX_Q[si]), si));
        tests.push_back(gen_merge(randAlmost(MAX_N[si]), randAlmost(MAX_Q[si]), si));

        if (si > 0) {
            tests.push_back(gen_converge(MAX_N[si], MAX_Q[si], si));
            tests.push_back(gen_converge(randAlmost(MAX_N[si]), randAlmost(MAX_Q[si]), si));
        }

        rep(ti, 0, tests.size()) {
            stringstream ss;
            ss << "tests/" << setfill('0') << setw(2) << (si + 1) << "_" << setw(2)
               << (ti + 1) << ".in";
            ofstream of(ss.str());
            tests[ti].check(si);
            tests[ti].print(of);
            cerr << ss.str() << " " << tests[ti].name << endl;
        }
    }
}
